function vo = calc_cone(agent, obst, d_safe, d_rvo, t)
% Calculate RVO from agent to obstacle

    r = obst.get_radius() + d_safe + d_rvo; % Obst radius with safety dist
    
    a_pos = agent.state(1:2,t);     % Agent position
    o_pos = obst.state(1:2,t);      % Obstacle position
    
    a_v = agent.state_dot(1:2,t-1); % Agent velocity
    v_v = obst.state_dot(1:2,t-1);  % Obstacle velocity

    
    % Calc tangent points from pos to obstacle + d_safe
    P = a_pos' - o_pos';
    d2 = dot(P,P);
    Q0 = o_pos'+r^2/d2*P;
    if d2-r^2 > 0
        T = r/d2*sqrt(d2-r^2)*P*[0,1;-1,0];
        p1 = Q0-T;  % Tangent points left
        p2 = Q0+T;  % Tangent points right

        % Move apex with respect to obstacle speed
        % Where finished indicate a passive obstacle, equal to VO 
        if obst.finished
            rvo_v = v_v;
        else
            rvo_v = (a_v+v_v)/2;
        end
        apex = a_pos + rvo_v; 
        p1 = p1' + rvo_v;
        p2 = p2' + rvo_v;

        % Calc each tangent line to obstacle through apex
        ang1 = vector_ang(apex,p1);
        ang2 = vector_ang(apex,p2);

        vo = cone(apex, [ang1; ang2]); % Class to save RVO 
    else
        % Corner case where calc above result in complex numbers
        % RVO is simplified to a halv plane in the direction of the obst
        ang = vector_ang(a_pos,o_pos);
        if obst.finished
            rvo_v = v_v;
        else
            rvo_v = (a_v+v_v)/2;
        end
        apex = a_pos + rvo_v;
        vo = cone(apex, [ang+1.5; ang-1.5]); % Class to save RVO
    end

end

