function vo = calc_cone_VO(agent, vehicle, d_safe, t)
% Calculate cone from pos tanget to a circle vehicle
    r = vehicle.get_radius() + d_safe;
    
    v_pos = vehicle.state(1:2,t);
    v_speed = vehicle.get_speed();
    v_psi = vehicle.state(3,t);
    
    a_pos = agent.state(1:2,t);
    a_speed = agent.get_speed();
    a_psi = agent.state(3,t);
    
    % Calc tangent points from pos to vehicle + d_safe
    P = a_pos' - v_pos';
    d2 = dot(P,P);
    Q0 = v_pos'+r^2/d2*P;
    T = r/d2*sqrt(d2-r^2)*P*[0,1;-1,0];
    p1 = Q0-T;
    p2 = Q0+T;
    
    % Move apex with respect to vehicle speed
    dx = v_speed*cos(v_psi);
    dy = v_speed*sin(v_psi);
    apex = [a_pos(1) + dx; a_pos(2) + dy];
    p1 = p1' + [dx; dy];
    p2 = p2' + [dx; dy];
    
    % Calc each tangent line to vehicle through apex
    ang1 = vector_ang(apex,p1);
    ang2 = vector_ang(apex,p2);
    
    vo = cone(apex, [ang1; ang2]);
end

