%% initiation
global database;

% CONSTS
TARGET_RADIUS = 3;                  % Start to break when in side TARGET_RADIUS
sens_rng = database.obstacleSenseDistance;

for o = 1 : length(obstacle_list)
    obstacle_list(o).finished = 1;
end

psi_d = zeros(1,time.get_sample_count);
f_count=1;
num_agents_finished = 0;
cluttered = 0;

%% Main control loop
for t=2:time.get_sample_count()-2
    state_update = zeros(2,length(agent_list));
    
    for ii = 1:length(agent_list)
        % Current agent state variables
        current_agent = agent_list(ii);
        current_position = current_agent.state(1:2,t);
        current_velo = current_agent.state_dot(1:2,t-1);
        current_speed = current_agent.get_speed();
        psi = current_agent.state(3,t);
        rvo_list = [];      % RVO
        conflict_list = [];
        
        % Skip if agent has crashed or reashed its goal
        if current_agent.finished
            if current_agent.get_max_speed() ~= 0
                current_agent.calculate_next_state(t,current_agent.state(3,t), 0);
                if current_agent.get_speed() == 0
                    current_agent.set_max_surge_speed(0);
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                end
            end
            continue;
        end     
        
        %% Agents
        for k = 1:length(agent_list)
            if k ~= ii && ~agent_list(k).finished
                % Check for collision with other agent
                dist_to_agent = agent_list(k).get_distance_to_point(t, current_position)...
                                - current_agent.get_d_safe();
                
                % Collision between agents
                if dist_to_agent < 0
                    disp('---- COLLISION ----')
                    if cluttered 
                        AAnoValidV = AAnoValidV + 1;
                    elseif agent_list(k).finished
                        collisionWithFinished = collisionWithFinished + 1;
                    elseif t < INIT_SAFE/(ASPEED*time_step)
                        AAhardInit = AAhardInit + 1;
                    end
                    
                    sim_data(sim_count,2) = -1;
                    num_agents_finished = num_agents_finished + 1;
                    current_agent.set_max_surge_speed(0);
                    current_agent.finished = true; 
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                    return;
                end
                
                % If agent is inside sensor range, find RVO
                if dist_to_agent < sens_rng
                    rvo = calc_cone(current_agent, agent_list(k), current_agent.get_d_safe(), D_rvo, t);
                    rvo_list = [rvo_list rvo];
                    conflict_list = [conflict_list agent_list(k)];
                end
            
            end
        end % End k in agent_list
        
        %% Obstacles
        for o = 1:length(obstacle_list)

            dist_to_obst = obstacle_list(o).get_distance_to_point(t, current_position) ...
                                - agent_list(ii).get_d_safe();
            
            if dist_to_obst < 0
                disp('--- COLLSION WITH OBSTACLE ---');
                sim_data(sim_count,2) = -1;
                num_agents_finished = num_agents_finished + 1;
                current_agent.set_max_surge_speed(0);
                current_agent.finished = true; 
                steps_left = length(t+1:time.get_sample_count());
                current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                break;
            end
            
            % If agent is inside sensor range, find RVO
            if dist_to_obst < sens_rng
                rvo = calc_cone(current_agent, obstacle_list(o), current_agent.get_d_safe(), D_rvo, t);
                rvo_list = [rvo_list rvo];
                conflict_list = [conflict_list obstacle_list(o)];
            end

        end
        
        %% Guidance
        
        % Compute desired velocity(DV) to target assuming there are no other agents
        [psi_d, Vd] = RVO_guidance(current_agent, t);
        
        % Check if DV is in RVO set
        conflict = 0;
        for kk = 1:size(rvo_list,2)         
            if rvo_list(kk).is_inside(current_position, Vd)
               conflict = 1;
            end
        end

        % If DV in VOs
        % Find v' that is outside RVO and closes to v_pref, within velocity restrictions
        if conflict && t > 2
            [Vd, noV] = choose_new_v(current_agent, Vd, rvo_list, t);
            if noV 
                cluttered = 1;
            end
        end
        
        % Stop if we are close enough
        if norm(current_agent.get_target() - current_agent.state(1:2,t))<=TARGET_RADIUS
            num_agents_finished = num_agents_finished + 1;
            current_agent.finished = true; 
            current_agent.calculate_next_state(t,current_agent.state(3,t), 0);
        end

        
        % Save desired speed and heading
        current_agent.V_d = Vd;
        state_update(:,ii) = Vd;
        
    end % End ii in agent_list
    
    % Update agent state
    for aa = 1:length(agent_list)
        if ~agent_list(aa).finished()
           [psi,speed] = cart2pol(state_update(1,aa),state_update(2,aa));
           agent_list(aa).calculate_next_state(t, psi, speed); 
        end
    end
    
    % Update obstacle state
    for oo = 1:length(obstacle_list)
        obstacle_list(oo).calculate_next_state(t, obstacle_list(oo).state(3,t), obstacle_list(oo).get_speed());
    end 
    
    % Plot an instant of the simulation
%     if(mod(t,plot_time_step)==0)
%         clf('reset');
%         plotInstantAgents(agent_list, obstacle_list,t, h_instant, time, axis_array);
%         hold on
%         %plot_all_vo(rvo_list, 'b')
%         drawnow;
%         movie_frames(f_count) = getframe(gcf);
%         f_count = f_count+1;
%     end
%     
    %% Stop when all agents are finished
    if num_agents_finished == length(agent_list)
        sim_data(sim_count,:) = [1, time.get_time(t)];
        return;
    end
    
end
