function psi_ca = collision_avoidance(M, psi)
% Look for the open interval that is closes to psi
% Compute C, the midle, of that interval

PI = 3.1416;
A1 = min(abs(M(1,:)));
A2 = min(abs(M(2,:)));

if size(M,2) == 1 
    psi_ca = mod(psi + PI/2,2*PI); 
    return 
end

if A1 < A2
    A1_i = find(abs(M(1,:)) == A1);
    if length(A1_i)>1
        A1_i = A1_i(2);
    end
    candidates = find(M(2,:)<M(1,A1_i));    % Find start of open interval
    A1_start = max(M(2,candidates));
    C = (M(1,A1_i)+A1_start)/2;     % Find middle of open interval
else
    A2_i = find(abs(M(2,:)) == A2);
    if length(A2_i)>1
        A2_i = A2_i(1);
    end
    candidates = find(M(1,:)>M(2,A2_i));    % Find end of open interval
    A2_end = min(M(1,candidates));
    C = (M(2,A2_i)+A2_end)/2;       % Find middle of open interval
end

% Choose middle of closest open interval as next desired heading
psi_ca = mod(psi + C,2*PI);
end