%% initiation

% CONSTS
ROR_COUNTDOWN = 15; % Number of timesteps agent is in ROR after conflicting agent is outside ROR_AREA
TARGET_RADIUS = 3;                  % Start to break when in side TARGET_RADIUS

% h_instant = figure('units','normalized','outerposition',[0.2 0.2 0.6 0.7]); % Figure for realtime plotting
% hold on;

psi_d = zeros(1,time.get_sample_count);
f_count=1;
num_agents_finished = 0;
shortest_agent_dist_plot = inf*ones(1, time.get_sample_count());
dist_to_obst_plot = inf*ones(length(agent_list), time.get_sample_count);
shortest_dist_to_agent = inf*ones(length(agent_list), time.get_sample_count);
ROR_active = zeros(1,length(agent_list));

%% Main control loop
for t=2:time.get_sample_count()-2
    
    for ii = 1:length(agent_list)
        current_agent = agent_list(ii);
        current_position = current_agent.state(1:2,t);
        
        
        % Skip if agent has crashed or reashed its goal
        if current_agent.finished
            if current_agent.get_max_speed() ~= 0
                current_agent.calculate_next_state(t,current_agent.state(3,t), 0);
                if current_agent.get_speed() == 0
                    current_agent.set_max_surge_speed(0);
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                end
            end
            continue;
        end
        
        
        %% Agents
        psi = current_agent.state(3,t);
        obst_inside_sensor_disk = [];
        cs = current_agent.get_sensor_disk_center(database.obstacleSenseDistance, t);
        for k = 1:length(agent_list)
            if k ~= ii
                % Check for collision with other agent
                dist_to_agent = agent_list(k).get_distance_to_point(t, current_position)...
                                - agent_list(ii).get_d_safe();
                
                % Save shortest distance between two agents for each
                % timestep
                if dist_to_agent < shortest_agent_dist_plot(t)
                    shortest_agent_dist_plot(t) = dist_to_agent;
                end
                
                if dist_to_agent < shortest_dist_to_agent(ii,t)
                    shortest_dist_to_agent(ii,t) = dist_to_agent;
                    closest_agent = k;
                end
                
                % Collision between agents
                if dist_to_agent < 0
                    disp('---- COLLISION ----')
                    num_agents_finished = num_agents_finished + 1;
                    current_agent.set_max_surge_speed(0);
                    current_agent.finished = true; 
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                    break;
                end
                
                % Check if other agent is inside sensor disk
                dist_to_cs = agent_list(k).get_distance_to_point(t, cs) ...
                                - agent_list(ii).get_d_safe(); 
                if dist_to_cs < database.obstacleSenseDistance/2
                    obst_inside_sensor_disk = [obst_inside_sensor_disk agent_list(k)];
                    if k == closest_agent 
                        closest_agent = length(obst_inside_sensor_disk);
                    end
                end
            end
        end
        
        %% Obstacles
        % Check for obstacles inside agents sensor range
        for o = 1:length(obstacle_list)
            dist_to_cs = obstacle_list(o).get_distance_to_point(t, cs) ...
                                - agent_list(ii).get_d_safe();
            dist_to_obst = obstacle_list(o).get_distance_to_point(t, current_position) ...
                                - agent_list(ii).get_d_safe();
            
            % Save shortest dist to obst
            if dist_to_obst < dist_to_obst_plot(ii, t)
                dist_to_obst_plot(ii, t) = dist_to_obst;
                closest_obst = o;
            end
            
            if dist_to_obst < 0
                disp('--- COLLSION WITH OBSTACLE ---');
                num_agents_finished = num_agents_finished + 1;
                current_agent.set_max_surge_speed(0);
                current_agent.finished = true; 
                steps_left = length(t+1:time.get_sample_count());
                current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                break;
            end
            
            if dist_to_cs < database.obstacleSenseDistance/2
                obst_inside_sensor_disk = [obst_inside_sensor_disk obstacle_list(o)];
                if o == closest_obst 
                    closest_obst = length(obst_inside_sensor_disk);
                end
            end
        end
        
        %% Guidance
        psi_guid = guidanceController(current_agent.get_target(), current_agent.state(:,t));
        new_speed = current_agent.get_max_speed();
        isCollisionMode = false;
        if ~isempty(obst_inside_sensor_disk)
            [M, M_pos, ROR] = get_M_extended(current_agent, obst_inside_sensor_disk, database.obstacleSenseDistance, t);
            %M = get_M(current_agent, obst_inside_sensor_disk, database.obstacleSenseDistance, t);  % Uncompensated M function
            try
                isCollisionMode = true;
                psi_d(t) = collision_avoidance(M, psi);
            catch
                disp('ERROR in collision_avoidance(M, psi)')
                psi_d(t) = psi_d(t-1); % Quick fix
            end
        else
             psi_d(t) = psi_guid;
             ROR = 0;
        end
         
        %%% Breaking rule
        if ROR == 1 
            ROR_active(ii) = ROR_active(ii) + 2; 
            if ROR_active(ii) > ROR_COUNTDOWN
                ROR_active(ii) = ROR_COUNTDOWN;
            end
            new_speed = MIN_SPEED * current_agent.get_max_speed();
        elseif ROR_active(ii) > 0 && ROR == 0
            ROR_active(ii) = ROR_active(ii) - 1;
            new_speed = MIN_SPEED * current_agent.get_max_speed();
        end
        %%%
        
        % Stop if we are close to target
        if norm(current_agent.get_target() - current_agent.state(1:2,t))<=TARGET_RADIUS
            num_agents_finished = num_agents_finished + 1;
            current_agent.finished = true; 
        end
        
        % Update states
        current_agent.calculate_next_state(t,psi_d(t), new_speed);
    end
    
    % Update obstacle posistion
    for oo = 1:length(obstacle_list)
        obstacle_list(oo).calculate_next_state(t, obstacle_list(oo).state(3,t), obstacle_list(oo).get_speed());
    end 
    
    % Plot an instant of the simulation
%     if(mod(t,plot_time_step)==0)
%         clf('reset');
%         plotInstantAgents(agent_list, obstacle_list, t, h_instant, time, axis_array);
%         drawnow;
%         movie_frames(f_count) = getframe(gcf);
%         f_count = f_count+1;
%     end
    

    
    %% Stop when all agents are finished
    if num_agents_finished == length(agent_list)
        disp('All agents finished')
        return;
    end
    
end
