function M = get_M(vehicle, obst_list, d_sensor, t)
% obst_list is a list of all obstacles that intersect with the sensor disk
global database;

M = [];
sensor_res = database.sensor_res;        % Sensor resolution in radians

yv = vehicle.state(1,t);  % Vehicle y-coordinate
xv = vehicle.state(2,t);  % Vehicle x-coordinate
psi = vehicle.state(3,t); % Heading

% Create a vector of angles to check
alpha = -pi/2 : sensor_res : pi/2;

% For each obstacle check what angles on the sensor disk that is blocked
for k = 1:length(obst_list)
    
    if obst_list(k).is_rectangle()
        %% Rectangular objects
        first = 0;
        last = pi/2;
       
        v = obst_list(k).get_vertices(t, vehicle.get_d_safe());
        l1 = [v(1,:) v(2,:)];
        l2 = [v(2,:) v(3,:)];
        l3 = [v(3,:) v(4,:)];
        l4 = [v(4,:) v(1,:)];
        lines = [l1; l2; l3; l4];

        % Test variables
        x1 = 0; x2 = 0; y1 = 0; y2 = 0;

        for i = 1:length(alpha)
            theta = mod(alpha(i)+psi, 2*pi);
            x = xv + d_sensor*sin(theta)*cos(alpha(i));  % x-coord on the sensor disk at angle theta
            y = yv + d_sensor*cos(theta)*cos(alpha(i));  % y-coord on the sensor disk at angle theta
            %plot([x xv], [y yv]);

            does_intersect = 0;
            for j = 1:4
               out = lineSegmentIntersect(lines(j,:),[yv, xv, y, x]);
               if out.intAdjacencyMatrix == 1
                   does_intersect = 1;
                   break;
               end
            end

            if does_intersect && first == 0
                first = alpha(i);
                y1 = y; x1 = x; % used for plotting
            elseif ~does_intersect && first ~= 0 
                last = alpha(i);
                y2 = y; x2 = x; % used for plotting
                break;
            end
        end
        
        
        
        
    else
        %% Circle objects
        r_obs = obst_list(k).get_radius() + vehicle.get_d_safe();
        c_obs = obst_list(k).state(1:2,t)'; % obstacle center

        first = 0;
        last = pi/2;

        % Test variables
        x1 = 0; x2 = 0; y1 = 0; y2 = 0;

        for i = 1:length(alpha)
            theta = mod(alpha(i)+psi, 2*pi);
            x = xv + d_sensor*sin(theta)*cos(alpha(i));  % x-coord on the sensor disk at angle theta
            y = yv + d_sensor*cos(theta)*cos(alpha(i));  % y-coord on the sensor disk at angle theta
            %plot([x xv], [y yv]);

            dist = dist_to_line([yv, xv], [y, x], c_obs);

            if dist < r_obs && first == 0
                first = alpha(i);
                y1 = y; x1 = x; % used for plotting
            elseif dist > r_obs && first ~= 0 
                last = alpha(i);
                y2 = y; x2 = x; % used for plotting
                break;
            end
        end
    end

    %plot([x1 xv], [y1 yv], 'k');
    %plot([x2 xv], [y2 yv], 'k');
    M = [M, [first last]'];
   
end

    % Sort M such that the first angles are in rising order
    M = sort(M,2);
 
    % Merge overlapping areas
    i = 1;
    len = size(M,2);
    while i < len
        if M(2,i) >= M(1,i+1)
            if len > i + 1
                M = [[M(1,i) M(2,i+1)]' M(:,i+2:len)];
            else
                M = [M(1,i) M(2,i+1)]';               
            end
            i = 0;
        end
        i = i + 1;
        len = size(M,2);
    end

    % Make sure first value of M is -pi/2
    if M(1,1) ~= -pi/2
        M = [-[pi/2 pi/2-0.001]' M];
    end
    % Make sure last value of M is pi/2
    if M(end,end) ~= pi/2
        M = [M [pi/2-0.001 pi/2]'];
    end
end