%% initiation

f_count=1;
movie_frames = struct('cdata',[],'colormap',[]);
psi_d = zeros(1,time.get_sample_count);
num_agents_finished = 0;
ROR_active = zeros(1,length(agent_list));

ROR_COUNTDOWN = 15; % Number of timesteps agent is in ROR after conflicting agent is outside ROR_AREA
TARGET_RADIUS = 3;                  % Start to break when in side TARGET_RADIUS


%% Main control loop
for t=2:time.get_sample_count()-2
    shortest_dist_to_agent = inf*ones(length(agent_list));
    shortest_dist_to_obst = inf*ones(length(agent_list));
    
    for ii = 1:length(agent_list)
        current_agent = agent_list(ii);
        current_position = current_agent.state(1:2,t);
        
        % Skip if agent has crashed or reashed its goal
        if current_agent.finished
            if current_agent.get_max_speed() ~= 0
                current_agent.calculate_next_state(t,current_agent.state(3,t), 0);
                if current_agent.get_speed() == 0
                    current_agent.set_max_surge_speed(0);
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                end
            end
            continue;
        end
             
        %% AGENTS - Guidance
        psi = current_agent.state(3,t);
        obst_inside_sensor_disk = [];
        cs = current_agent.get_sensor_disk_center(database.obstacleSenseDistance, t);
        for k = 1:length(agent_list)
            if k ~= ii
                % Check for collision with other agent
                dist_to_agent = agent_list(k).get_distance_to_point(t, current_position)...
                                - agent_list(ii).get_d_safe();
                
                % Save shortest distance between two agents for each
                % timestep
                %if dist_to_agent < shortest_agent_dist(t)
                %    shortest_agent_dist(t) = dist_to_agent;
                %end
                
                if dist_to_agent < shortest_dist_to_agent(ii)
                    shortest_dist_to_agent(ii) = dist_to_agent;
                    closest_agent = k;
                end
                                            
                if dist_to_agent < 0
                    disp('---- COLLISION ----')
                    sim_data(sim_count,2) = -1;
                    num_agents_finished = num_agents_finished + 1;
                    current_agent.set_max_surge_speed(0);
                    current_agent.finished = true; 
                    steps_left = length(t+1:time.get_sample_count());
                    current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
                    return;
                end
                
                % Check if other agent is inside sensor disk
                dist_to_cs = agent_list(k).get_distance_to_point(t, cs) ...
                                - agent_list(ii).get_d_safe(); 
                if dist_to_cs < database.obstacleSenseDistance/2
                    agentEncounters(sim_count) = 1;
                    obst_inside_sensor_disk = [obst_inside_sensor_disk agent_list(k)];
                    if k == closest_agent 
                        closest_agent = length(obst_inside_sensor_disk);
                    end
                end
            end
        end
        
        %% OBSTACLES - Check for obstacles inside agents sensor range
        for o = 1:length(obstacle_list)
            dist_to_cs = obstacle_list(o).get_distance_to_point(t, cs) ...
                                - agent_list(ii).get_d_safe();
            dist_to_obst = obstacle_list(o).get_distance_to_point(t, current_position) ...
                                - agent_list(ii).get_d_safe();
            
            % Save shortest dist to obst
            if dist_to_obst < shortest_dist_to_obst(ii)
                shortest_dist_to_obst(ii) = dist_to_obst;
                closest_obst = o;
            end                
                            
            if dist_to_obst < 0
                disp('--- COLLSION WITH OBSTACLE ---');
                sim_data(sim_count,2) = -1;
                return;
            end
            
            if dist_to_cs < database.obstacleSenseDistance/2
                obstEncounters(sim_count) = 1;
                obst_inside_sensor_disk = [obst_inside_sensor_disk obstacle_list(o)];
                if o == closest_obst 
                    closest_obst = length(obst_inside_sensor_disk);
                end
            end
        end
        
        %% COLLISION AVOIDANCE
        isCollisionMode = false;
        if ~isempty(obst_inside_sensor_disk)
            [M, M_pos, ROR] = get_M_extended(current_agent, obst_inside_sensor_disk, database.obstacleSenseDistance, t);
            %M = get_M(current_agent, obst_inside_sensor_disk, database.obstacleSenseDistance, t);  % Uncompensated M function
            %ROR = 0;
            try
                isCollisionMode = true;
                psi_d(t) = collision_avoidance(M, psi);
            catch
                disp('-------------------------- WARNING -------------------------')
                disp('Something went terribly wrong in collision_avoidance(M, psi)')
                disp('------------------------------------------------------------')
                sim_data(sim_count,2) = -1;
                return;
            end
        else
             psi_d(t) = guidanceController(current_agent.get_target(), current_agent.state(:,t));
             ROR = 0;
        end
        
        % Ajust speed if pointing away from target
        new_speed = current_agent.get_max_speed();
         
        if ROR == 1 
            ROR_active(ii) = ROR_active(ii) + 2; 
            if ROR_active(ii) > ROR_COUNTDOWN
                ROR_active(ii) = ROR_COUNTDOWN;
            end
            new_speed = MIN_SPEED * current_agent.get_max_speed();
        elseif ROR_active(ii) > 0 && ROR == 0
            ROR_active(ii) = ROR_active(ii) - 1;
            new_speed = MIN_SPEED * current_agent.get_max_speed();
        end
        
        % Stop if we are close enough
        if norm(current_agent.get_target() - current_agent.state(1:2,t))<=TARGET_RADIUS
            num_agents_finished = num_agents_finished + 1;
            current_agent.finished = true; 
            %current_agent.set_max_surge_speed(0);
            %steps_left = length(t+1:time.get_sample_count());
            %current_agent.state(:,t+1:time.get_sample_count()) = diag(current_agent.state(:,t))*ones(4,steps_left);
        end
        
        % Update states
        current_agent.calculate_next_state(t,psi_d(t), new_speed);
    end
    
    % Update obstacle posistion
    for oo = 1:length(obstacle_list)
        obstacle_list(oo).calculate_next_state(t, obstacle_list(oo).state(3,t), obstacle_list(oo).get_speed());
    end 
    
    
    %Plot an instant of the simulation
%     if(mod(t,plot_time_step)==0)
%         clf('reset');
%         plotInstantAgents(agent_list, obstacle_list, t, h_instant, time, axis_array);
%         drawnow;
%         movie_frames(f_count) = getframe(gcf);
%         f_count = f_count+1;
%     end
    

    %% Stop when all agents are finished
    if num_agents_finished == length(agent_list)
        %disp('All agents finished')
        sim_data(sim_count,:) = [1, time.get_time(t)]; 
        return;
    end
    
end
