clear all;
close all;
clc;

addpath('Plotting');
addpath('obstacles');
addpath('RVO');

global database;

%% Variable parameters
RNG = 530;        % Random seed
NSIM = 1000;      % Number of simulations 
INIT_SAFE = 3;    % Distance between init_pos or target pos
AXIS = 50;        % Square area for init and target position
NAGENTS = 10;     % Number of agents
NOBST = 0;        % Number of obstacles
ASPEED = 3;       % Agent max speed
MIN_SPEED = 0.4;  % Min agent speed in percentage of max speed
OSPEED = 2;       % Obstacle speed
R_MAX = 1;        % Agent r_max
D_rvo = 0.5;      % Safety margin in calc_cone()
TSTOP = 60;       % Max simulation time
RVO = 0;          % Use RVO algorithm when true

%% Initialize global variables;

database.obstacleSafetyDistance = 1;
database.obstacleSenseDistance = 7;
database.sensor_res = 0.04;        % Sensor resolution in radians



%% Time variables
time_start = 0;
time_stop = TSTOP;
time_step = 0.05;
time=sim_time(time_start, time_stop, time_step);

%% Plotting
% Plot an instant each Nth time step (typically 5 for movie making, 50 for
plot_time_step = 25;
h_instant = figure('units','normalized','outerposition',[0.2 0.2 0.6 0.7]); % Figure for realtime plotting
hold on;

frame_count = ceil(time.get_sample_count()/plot_time_step);

%% Run Monte Carlo simulation
num_of_simulations = NSIM;
total_agents = NAGENTS;
total_obst = NOBST;
sim_data = zeros(num_of_simulations, 2);
agentEncounters = zeros(num_of_simulations, 1);
obstEncounters = zeros(num_of_simulations, 1);
AAnoValidV = 0;
collisionWithFinished = 0;
AAhardInit = 0;

agent_speed = ASPEED;
obst_speed = OSPEED;
max_yaw_rate = R_MAX;
radius = 1;
obst_r = 1;
d_safe = database.obstacleSafetyDistance;

yArea = [0 AXIS];
xArea = [0 AXIS];
axis_array = [[-10 yArea(2)+10] [-10 xArea(2)+10]]; %[yArea xArea]; 
init_dist = 2*max(radius,obst_r) + INIT_SAFE*d_safe;

test11 = zeros(num_of_simulations,1);
test22 = zeros(num_of_simulations,1);

rng(RNG) % Save seed to make sure diffrent algorithms run on same scenarios
for sim_count = 1:num_of_simulations
    disp(['Simulation ', num2str(sim_count)])
    init_xpos = randi(xArea, 1, 20);
    init_ypos = randi(yArea, 1, 20);
    %%% Only for moving obst %%%
    %init_xpos((total_agents+1):(total_agents+total_obst)) = [10 20 30 40];
    %init_ypos((total_agents+1):(total_agents+total_obst)) = [0 50 0 50];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    safe_guard = 0;
    while resolve_overlapping_pos(init_xpos(1:(total_agents+total_obst)),...
                         init_ypos(1:(total_agents+total_obst)), init_dist)
        init_xpos = randi(xArea, 1, 20);
        init_ypos = randi(yArea, 1, 20);
        %%% Only for moving obst %%%
        %init_xpos((total_agents+1):(total_agents+total_obst)) = [10 20 30 40];
        %init_ypos((total_agents+1):(total_agents+total_obst)) = [0 50 0 50];
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %disp('--- Init pos overlap ---')
        safe_guard = safe_guard + 1;
        test11(sim_count) = safe_guard;
        if safe_guard > 50 
            disp('Not able to find a feasible set of initial pos')
            continue;
        end
    end
    init_ang  = 2*pi*rand(1,20);
    %%% Only for moving obst %%%
    %init_ang((total_agents+1):(total_agents+total_obst)) = [pi/2 -pi/2 pi/2 -pi/2];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    safe_guard = 0;
    init_list = [[init_xpos(1), init_ypos(1), init_ang(1), 0], [0, 0, 0, 0];
                 [init_xpos(2), init_ypos(2), init_ang(2), 0], [0, 0, 0, 0];
                 [init_xpos(3), init_ypos(3), init_ang(3), 0], [0, 0, 0, 0];
                 [init_xpos(4), init_ypos(4), init_ang(4), 0], [0, 0, 0, 0];
                 [init_xpos(5), init_ypos(5), init_ang(5), 0], [0, 0, 0, 0];
                 [init_xpos(6), init_ypos(6), init_ang(6), 0], [0, 0, 0, 0];
                 [init_xpos(7), init_ypos(7), init_ang(7), 0], [0, 0, 0, 0];
                 [init_xpos(8), init_ypos(8), init_ang(8), 0], [0, 0, 0, 0];
                 [init_xpos(9), init_ypos(9), init_ang(9), 0], [0, 0, 0, 0];
                 [init_xpos(10), init_ypos(10), init_ang(10), 0], [0, 0, 0, 0];
                 [init_xpos(11), init_ypos(11), init_ang(11), 0], [0, 0, 0, 0];
                 [init_xpos(12), init_ypos(12), init_ang(12), 0], [0, 0, 0, 0];
                 [init_xpos(13), init_ypos(13), init_ang(13), 0], [0, 0, 0, 0];
                 [init_xpos(14), init_ypos(14), init_ang(14), 0], [0, 0, 0, 0];
                 [init_xpos(15), init_ypos(15), init_ang(15), 0], [0, 0, 0, 0];
                 [init_xpos(16), init_ypos(16), init_ang(16), 0], [0, 0, 0, 0];
                 [init_xpos(17), init_ypos(17), init_ang(17), 0], [0, 0, 0, 0];
                 [init_xpos(18), init_ypos(18), init_ang(18), 0], [0, 0, 0, 0];
                 [init_xpos(19), init_ypos(19), init_ang(19), 0], [0, 0, 0, 0];
                 [init_xpos(20), init_ypos(20), init_ang(20), 0], [0, 0, 0, 0];
                         ];
    tx = randi(xArea, 1, 15);
    ty = randi(yArea, 1, 15);
    safe_guard = 0;
    txx = [tx(1:total_agents) init_xpos((total_agents+1):(total_agents+total_obst))];
    tyy = [ty(1:total_agents) init_ypos((total_agents+1):(total_agents+total_obst))];
    while resolve_overlapping_pos(txx, tyy, init_dist)
        tx = randi(xArea, 1, 15);
        ty = randi(yArea, 1, 15);
        txx = [tx(1:total_agents) init_xpos((total_agents+1):(total_agents+total_obst))];
        tyy = [ty(1:total_agents) init_ypos((total_agents+1):(total_agents+total_obst))];
        %disp('--- Target overlap ---')
        safe_guard = safe_guard + 1;
        test22(sim_count) = safe_guard;
        if safe_guard > 50 
            disp('Not able to find a feasible set of target pos')
            continue;
        end
    end
    param_agents = [
        tx(1), ty(1), radius, d_safe, agent_speed, max_yaw_rate;
        tx(2), ty(2), radius, d_safe, agent_speed, max_yaw_rate;
        tx(3), ty(3), radius, d_safe, agent_speed, max_yaw_rate;
        tx(4), ty(4), radius, d_safe, agent_speed, max_yaw_rate;
        tx(5), ty(5), radius, d_safe, agent_speed, max_yaw_rate;
        tx(6), ty(6), radius, d_safe, agent_speed, max_yaw_rate;
        tx(7), ty(7), radius, d_safe, agent_speed, max_yaw_rate;
        tx(8), ty(8), radius, d_safe, agent_speed, max_yaw_rate;
        tx(9), ty(9), radius, d_safe, agent_speed, max_yaw_rate;
        tx(10), ty(10), radius, d_safe, agent_speed, max_yaw_rate;
        tx(11), ty(11), radius, d_safe, agent_speed, max_yaw_rate;
        tx(12), ty(12), radius, d_safe, agent_speed, max_yaw_rate;
        tx(13), ty(13), radius, d_safe, agent_speed, max_yaw_rate;
        tx(14), ty(14), radius, d_safe, agent_speed, max_yaw_rate;
        tx(15), ty(15), radius, d_safe, agent_speed, max_yaw_rate;
        ];
    param_obst = [
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        0, 0, obst_r, d_safe, obst_speed, 0;
        ];
    agent_list = generate_agents(param_agents(1:total_agents,:),...
                    init_list(1:total_agents,:), time);
    obstacle_list = generate_agents(param_obst(1:total_obst,:),...
                init_list((total_agents+1):(total_agents+total_obst),:), time);     
    
    %% Run simulation
    if RVO 
        monte_do_sim_RVO;
    else
        monte_do_sim;
    end
    

end
numSuccess = sum(sim_data(:,1));
numCollision = sum(sim_data(:,2)==-1);
disp(['Success: ',num2str(100* numSuccess/num_of_simulations), '%'])
disp(['Collision: ', ...
    num2str(100* (numCollision)/num_of_simulations), '%'])
disp(['Time-out: ', num2str(100* (num_of_simulations-numSuccess-numCollision)/num_of_simulations), '%'])
tt = sum(sim_data(:,2));
avg_time = (tt + numCollision)/sum(sim_data(:,2)>0);
disp(['Average time: ', num2str(avg_time)])
noObstEn = num_of_simulations - sum(obstEncounters);
noAgentEn = num_of_simulations - sum(agentEncounters);
%disp(['Cases with no Obstacle encounters ', num2str(100* (noObstEn/num_of_simulations)), '%'])
%disp(['Cases with no Agent encounters ', num2str(100* (noAgentEn/num_of_simulations)), '%'])

